import requests
import json
from .result import GenericResult, SeasonalResult, OperatorResult, WeaponResult, WeaponCategoryResult
from .platforms import Platform
from .seasons import Seasons
from .exceptions import ApiError


def check_for_api_error(response: dict):
    if "status" in list(response.keys()):
        if response["status"] == "error":
            raise ApiError(response["error"])


class Stats:
    def __init__(self, key):
        self.key = key
        self.headers = {"Authorization": f"Bearer {self.key}",
                        "User-Agent": "r6stats.py/1.0.0"}

    def get_generic_stats(self, username, platform: Platform):

        request = requests.get(f'https://api2.r6stats.com/public-api/stats/{username}/{platform}/generic',
                               headers=self.headers)
        content = json.loads(request.content)

        check_for_api_error(content)

        return GenericResult(
            username=content["username"],
            platform=content["platform"],
            ubisoft_id=content["ubisoft_id"],
            uplay_id=content["uplay_id"],
            avatar_url=content["avatar_url_256"],
            last_updated=content["last_updated"],
            aliases=content["aliases"],
            level=content["progression"]["level"],
            lootbox_probability=content["progression"]["lootbox_probability"],
            total_xp=content["progression"]["total_xp"],
            assists=content["stats"]["general"]["assists"],
            barricades_deployed=content["stats"]["general"]["barricades_deployed"],
            blind_kills=content["stats"]["general"]["blind_kills"],
            bullets_fired=content["stats"]["general"]["bullets_fired"],
            bullets_hit=content["stats"]["general"]["bullets_hit"],
            dbnos=content["stats"]["general"]["dbnos"],
            deaths=content["stats"]["general"]["deaths"],
            distance_travelled=content["stats"]["general"]["distance_travelled"],
            draws=content["stats"]["general"]["draws"],
            gadgets_destroyed=content["stats"]["general"]["gadgets_destroyed"],
            games_played=content["stats"]["general"]["games_played"],
            headshots=content["stats"]["general"]["headshots"],
            kd=content["stats"]["general"]["kd"],
            kills=content["stats"]["general"]["kills"],
            losses=content["stats"]["general"]["losses"],
            melee_kills=content["stats"]["general"]["melee_kills"],
            penetration_kills=content["stats"]["general"]["penetration_kills"],
            playtime=content["stats"]["general"]["playtime"],
            rappel_breaches=content["stats"]["general"]["rappel_breaches"],
            reinforcements_deployed=content["stats"]["general"]["reinforcements_deployed"],
            revives=content["stats"]["general"]["revives"],
            suicides=content["stats"]["general"]["suicides"],
            wins=content["stats"]["general"]["wins"],
            wl=content["stats"]["general"]["wl"],
            casual_data=content["stats"]["queue"]["casual"],
            ranked_data=content["stats"]["queue"]["ranked"],
            unranked_data=content["stats"]["queue"]["other"],
            gamemode_bomb=content["stats"]["gamemode"]["bomb"],
            gamemode_secure=content["stats"]["gamemode"]["secure_area"],
            gamemode_hostage=content["stats"]["gamemode"]["hostage"]
        )

    def get_seasonal_stats(self, username, platform: Platform, season: Seasons):

        request = requests.get(f'https://api2.r6stats.com/public-api/stats/{username}/{platform}/seasonal',
                               headers=self.headers)
        content = json.loads(request.content)
        check_for_api_error(content)

        region = content["seasons"][season]["regions"]["ncsa"][0]

        return SeasonalResult(
            username=content["username"],
            platform=content["platform"],
            ubisoft_id=content["ubisoft_id"],
            uplay_id=content["uplay_id"],
            avatar_url=content["avatar_url_256"],
            last_updated=content["last_updated"],
            name=content["seasons"][season]["name"],
            start_date=content["seasons"][season]["start_date"],
            end_date=content["seasons"][season]["end_date"],
            abandons=region["abandons"],
            losses=region["losses"],
            max_rank=region["max_rank"],
            max_mmr=region["max_mmr"],
            mmr=region["mmr"],
            next_rank_mmr=region["next_rank_mmr"],
            prev_rank_mmr=region["prev_rank_mmr"],
            rank=region["rank"],
            skill_mean=region["skill_mean"],
            skill_standard_deviation=region["skill_standard_deviation"],
            created_for_date=region["created_for_date"],
            wins=region["wins"],
            kills=region["kills"],
            deaths=region["deaths"],
            last_match_mmr_change=region["last_match_mmr_change"],
            last_match_result=region["last_match_result"],
            last_match_skill_standard_deviation_change=region["last_match_skill_standard_deviation_change"],
            last_match_skill_mean_change=region["last_match_skill_mean_change"],
            champions_rank_position=region["champions_rank_position"],
            max_rank_text=region["max_rank_text"],
            max_rank_image=region["max_rank_image"],
            rank_image=region["rank_image"],
            rank_text=region["rank_text"]
        )

    def get_operator_stats(self, username, platform: Platform, operator: str):

        request = requests.get(f'https://api2.r6stats.com/public-api/stats/{username}/{platform}/operators',
                               headers=self.headers)
        name = None
        ctu = None
        dbnos = None
        abilities = None
        kd = None
        wl = None
        kills = None
        badge_image = None
        deaths = None
        experience = None
        role = None
        wins = None
        headshots = None
        losses = None
        melee_kills = None
        playtime = None

        content = json.loads(request.content)
        check_for_api_error(content)

        for op in content["operators"]:
            if op["name"] == operator.capitalize():
                name = op["name"]
                ctu = op["ctu"]
                dbnos = op["dbnos"]
                abilities = op["abilities"]
                kd = op["kd"]
                wl = op["wl"]
                kills = op["kills"]
                badge_image = op["badge_image"]
                deaths = op["deaths"]
                experience = op["experience"]
                role = op["role"]
                wins = op["wins"]
                headshots = op["headshots"]
                losses = op["losses"]
                melee_kills = op["melee_kills"]
                playtime = op["playtime"]

        return OperatorResult(
            username=content["username"],
            platform=content["platform"],
            ubisoft_id=content["ubisoft_id"],
            uplay_id=content["uplay_id"],
            avatar_url=content["avatar_url_256"],
            last_updated=content["last_updated"],
            name=name,
            ctu=ctu,
            dbnos=dbnos,
            abilities=abilities,
            kd=kd,
            wl=wl,
            kills=kills,
            badge_image=badge_image,
            deaths=deaths,
            experience=experience,
            role=role,
            wins=wins,
            headshots=headshots,
            losses=losses,
            melee_kills=melee_kills,
            playtime=playtime
        )

    def get_weapon_stats(self, username, platform: Platform, weapon: str):

        request = requests.get(f'https://api2.r6stats.com/public-api/stats/{username}/{platform}/weapons',
                               headers=self.headers)
        weapon_name = None
        category = None
        kills = None
        deaths = None
        kd = None
        headshots = None
        headshot_percentage = None
        times_chosen = None
        bullets_fired = None
        bullets_hit = None

        content = json.loads(request.content)
        check_for_api_error(content)

        for weapons in content["weapons"]:
            if weapons["name"] == weapon:
                weapon_name = weapons["name"]
                category = weapons["category"]
                kills = weapons["kills"]
                deaths = weapons["deaths"]
                kd = weapons["kd"]
                headshots = weapons["headshots"]
                headshot_percentage = weapons["headshot_percentage"]
                times_chosen = weapons["times_chosen"]
                bullets_fired = weapons["bullets_fired"]
                bullets_hit = weapons["bullets_hit"]

        return WeaponResult(
            username=content["username"],
            platform=content["platform"],
            ubisoft_id=content["ubisoft_id"],
            uplay_id=content["uplay_id"],
            avatar_url=content["avatar_url_256"],
            last_updated=content["last_updated"],
            weapon_name=weapon_name,
            category=category,
            kills=kills,
            deaths=deaths,
            kd=kd,
            headshots=headshots,
            headshot_percentage=headshot_percentage,
            times_chosen=times_chosen,
            bullets_fired=bullets_fired,
            bullets_hit=bullets_hit
        )

    def get_weapon_category_stats(self, username, platform: Platform, category: str):

        request = requests.get(f'https://api2.r6stats.com/public-api/stats/{username}/{platform}/weapons',
                               headers=self.headers)

        kills = None
        deaths = None
        kd = None
        headshots = None
        headshot_percentage = None
        times_chosen = None
        bullets_fired = None
        bullets_hit = None

        content = json.loads(request.content)
        check_for_api_error(content)

        for ca in content["categories"]:
            if ca["category"] == category:
                category = ca["category"]
                kills = ca["kills"]
                deaths = ca["deaths"]
                kd = ca["kd"]
                headshots = ca["headshots"]
                headshot_percentage = ca["headshot_percentage"]
                times_chosen = ca["times_chosen"]
                bullets_fired = ca["bullets_fired"]
                bullets_hit = ca["bullets_hit"]

        return WeaponCategoryResult(
            username=content["username"],
            platform=content["platform"],
            ubisoft_id=content["ubisoft_id"],
            uplay_id=content["uplay_id"],
            avatar_url=content["avatar_url_256"],
            last_updated=content["last_updated"],
            category=category,
            kills=kills,
            deaths=deaths,
            kd=kd,
            headshots=headshots,
            headshot_percentage=headshot_percentage,
            times_chosen=times_chosen,
            bullets_fired=bullets_fired,
            bullets_hit=bullets_hit
        )



from .exceptions import OperatorNotFoundError

class Operators:
    sledge = "Sledge"
    thatcher = "Thatcher"
    ash = "Ash"
    thermite = "Thermite"
    twitch = "Twitch"
    montagne = "Montagne"
    glaz = "Glaz"
    fuze = "Fuze"
    blitz = "Blitz"
    iq = "IQ"
    buck = "Buck"
    blackbeard = "Blackbeard"
    capitao = "Capitão"
    hibana = "Hibana"
    jackal = "Jackal"
    ying = "Ying"
    zofia = "Zofia"
    dokkaebi = "Dokkaebi"
    lion = "Lion"
    finka = "Finka"
    maverick = "Maverick"
    nomad = "Nomad"
    gridlock = "Gridlock"
    nokk = "Nøkk"
    amaru = "Amaru"
    kali = "Kali"
    iana = "Iana"
    ace = "Ace"
    zero = "Zero"
    flores = "Flores"
    smoke = "Smoke"
    mute = "Mute"
    castle = "Castle"
    pulse = "Pulse"
    doc = "Doc"
    rook = "Rook"
    kapkan = "Kapkan"
    tachanka = "Tachanka"
    jager = "Jäger"
    bandit = "Bandit"
    frost = "Frost"
    valkyrie = "Valkyrie"
    caveira = "Caveira"
    echo = "Echo"
    mira = "Mira"
    lesion = "Lesion"
    ela = "Ela"
    vigil = "Vigil"
    alibi = "Alibi"
    maestro = "Maestro"
    clash = "Clash"
    kaid = "Kaid"
    mozzie = "Mozzie"
    warden = "Warden"
    goyo = "Goyo"
    wamai = "Wamai"
    oryx = "Oryx"
    melusi = "Melusi"
    aruni = "Aruni"

    operators = ["Sledge", "Thatcher", "Ash", "Thermite", "Twitch", "Montagne", "Glaz", "Fuze", "Blitz", "IQ", "Buck",
                 "Blackbeard", "Capitão", "Hibana", "Jackal", "Ying", "Zofia", "Dokkaebi", "Lion", "Finka", "Maverick",
                 "Nomad", "Gridlock", "Nøkk", "Amaru", "Kali", "Iana", "Ace", "Zero", "Flores", "Defenders", "Recruit",
                 "Smoke", "Mute", "Castle", "Pulse", "Doc", "Rook", "Kapkan", "Tachanka", "Jäger", "Bandit", "Frost",
                 "Valkyrie", "Caveira", "Echo", "Mira", "Lesion", "Ela", "Vigil", "Alibi", "Maestro", "Clash", "Kaid",
                 "Mozzie", "Warden", "Goyo", "Wamai", "Oryx", "Melusi", "Aruni"]

    @classmethod
    def search(cls, query: str):
        matches = []
        for op in cls.operators:
            if op.startswith(query.capitalize()):
                matches.append(op)

        if not len(matches):
            raise OperatorNotFoundError("Your operator search returned no results")
        elif len(matches) > 1:
            raise OperatorNotFoundError("Your operator search returned multiple results")

        return matches[0]



class Seasons:
    crimson_heist = "crimson_heist"
    neon_dawn = "neon_dawn"
    shadow_legacy = "shadow_legacy"
    steel_wave = "steel_wave"
    void_edge = "void_edge"
    shifting_tides = "shifting_tides"
    ember_rise = "ember_rise"
    phantom_sight = "phantom_sight"
    burnt_horizon = "burnt_horizon"
    wind_bastion = "wind_bastion"
    grim_sky = "grim_sky"
    para_bellum = "para_bellum"
    chimera = "chimera"
    white_noise = "white_noise"
    blood_orchid = "blood_orchid"
    health = "health"



import setuptools

with open("README.md") as f:
    readme = f.read()

setuptools.setup(
    name="r6stats.py",
    version="1.0.1",
    author="MakufonSkifto",
    description="A Python wrapper for the R6Stats.com API",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/MakufonSkifto/r6stats.py",
    packages=setuptools.find_packages(),
    install_requires=["requests"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],
    keywords="r6stats api siege r6siege",
    python_requires='>=3.0',
)



class GenericResult:
    def __init__(self, username, platform, ubisoft_id, uplay_id, avatar_url, last_updated, aliases, level, total_xp,
                 lootbox_probability, assists, barricades_deployed, blind_kills, bullets_fired, dbnos, deaths, draws,
                 distance_travelled, gadgets_destroyed, games_played, headshots, kd, kills, losses, melee_kills, wl,
                 penetration_kills, playtime, rappel_breaches, reinforcements_deployed, revives, suicides, wins,
                 casual_data, ranked_data, unranked_data, gamemode_bomb, gamemode_secure, gamemode_hostage,
                 bullets_hit):
        self.username = username
        self.platform = platform
        self.ubisoft_id = ubisoft_id
        self.uplay_id = uplay_id
        self.avatar_url = avatar_url
        self.last_updated = last_updated
        self.aliases = aliases
        self.level = level
        self.total_xp = total_xp
        self.lootbox_probability = lootbox_probability
        self.assists = assists
        self.barricades_deployed = barricades_deployed
        self.blind_kills = blind_kills
        self.bullets_fired = bullets_fired
        self.dbnos = dbnos
        self.deaths = deaths
        self.draws = draws
        self.distance_travelled = distance_travelled
        self.gadgets_destroyed = gadgets_destroyed
        self.games_played = games_played
        self.headshots = headshots
        self.kd = kd
        self.kills = kills
        self.losses = losses
        self.melee_kills = melee_kills
        self.wl = wl
        self.penetration_kills = penetration_kills
        self.playtime = playtime
        self.rappel_breaches = rappel_breaches
        self.reinforcements_deployed = reinforcements_deployed
        self.revives = revives
        self.suicides = suicides
        self.wins = wins
        self.casual_data = casual_data
        self.ranked_data = ranked_data
        self.unranked_data = unranked_data
        self.gamemode_bomb = gamemode_bomb
        self.gamemode_secure = gamemode_secure
        self.gamemode_hostage = gamemode_hostage
        self.bullets_hit = bullets_hit


class SeasonalResult:
    def __init__(self, username, platform, ubisoft_id, uplay_id, avatar_url, last_updated, name, start_date, end_date,
                 max_mmr, max_rank, losses, mmr, next_rank_mmr, prev_rank_mmr, rank, skill_mean, wins, kills, deaths,
                 skill_standard_deviation, created_for_date, last_match_mmr_change, last_match_skill_mean_change,
                 last_match_skill_standard_deviation_change, last_match_result, champions_rank_position, rank_text,
                 rank_image, max_rank_text, max_rank_image, abandons):
        self.username = username
        self.platform = platform
        self.ubisoft_id = ubisoft_id
        self.uplay_id = uplay_id
        self.avatar_url = avatar_url
        self.last_updated = last_updated
        self.name = name
        self.start_date = start_date
        self.end_date = end_date
        self.max_mmr = max_mmr
        self.max_rank = max_rank
        self.losses = losses
        self.mmr = mmr
        self.next_rank_mmr = next_rank_mmr
        self.prev_rank_mmr = prev_rank_mmr
        self.rank = rank
        self.skill_mean = skill_mean
        self.wins = wins
        self.kills = kills
        self.deaths = deaths
        self.skill_standard_deviation = skill_standard_deviation
        self.created_for_date = created_for_date
        self.last_match_mmr_change = last_match_mmr_change
        self.last_match_skill_mean_change = last_match_skill_mean_change
        self.last_match_skill_standard_deviation_change = last_match_skill_standard_deviation_change
        self.last_match_result = last_match_result
        self.champions_rank_position = champions_rank_position
        self.rank_text = rank_text
        self.rank_image = rank_image
        self.max_rank_text = max_rank_text
        self.max_rank_image = max_rank_image
        self.abandons = abandons


class OperatorResult:
    def __init__(self, username, platform, ubisoft_id, uplay_id, avatar_url, last_updated, name, ctu, role, kills, kd,
                 deaths, wins, losses, wl, headshots, dbnos, melee_kills, experience, playtime, badge_image, abilities):
        self.username = username
        self.platform = platform
        self.ubisoft_id = ubisoft_id
        self.uplay_id = uplay_id
        self.avatar_url = avatar_url
        self.last_updated = last_updated
        self.name = name
        self.ctu = ctu
        self.role = role
        self.kills = kills
        self.kd = kd
        self.deaths = deaths
        self.wins = wins
        self.wl = wl
        self.headshots = headshots
        self.wins = wins
        self.losses = losses
        self.dbnos = dbnos
        self.melee_kills = melee_kills
        self.experience = experience
        self.playtime = playtime
        self.badge_image = badge_image
        self.abilities = abilities


class WeaponResult:
    def __init__(self, username, platform, ubisoft_id, uplay_id, avatar_url, last_updated, weapon_name, category, kills,
                 deaths, kd, headshots, headshot_percentage, times_chosen, bullets_fired, bullets_hit):
        self.username = username
        self.platform = platform
        self.ubisoft_id = ubisoft_id
        self.uplay_id = uplay_id
        self.avatar_url = avatar_url
        self.last_updated = last_updated
        self.weapon_name = weapon_name
        self.category = category
        self.kills = kills
        self.deaths = deaths
        self.kd = kd
        self.headshots = headshots
        self.headshot_percentage = headshot_percentage
        self.times_chosen = times_chosen
        self.bullets_fired = bullets_fired
        self.bullets_hit = bullets_hit


class WeaponCategoryResult:
    def __init__(self, username, platform, ubisoft_id, uplay_id, avatar_url, last_updated, category, kills,
                 deaths, kd, headshots, headshot_percentage, times_chosen, bullets_fired, bullets_hit):
        self.username = username
        self.platform = platform
        self.ubisoft_id = ubisoft_id
        self.uplay_id = uplay_id
        self.avatar_url = avatar_url
        self.last_updated = last_updated
        self.category = category
        self.kills = kills
        self.deaths = deaths
        self.kd = kd
        self.headshots = headshots
        self.headshot_percentage = headshot_percentage
        self.times_chosen = times_chosen
        self.bullets_fired = bullets_fired
        self.bullets_hit = bullets_hit
